var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';

import { TransitionMotion, spring } from 'react-motion';
import { colorMotionSpring, getInterpolatedColor } from '../../../lib/colors';
import { HeatMapPropTypes } from './props';
import enhance from './enhance';
import Container from '../Container';
import SvgWrapper from '../SvgWrapper';
import Grid from '../../axes/Grid';
import Axes from '../../axes/Axes';
import HeatMapCellRect from './HeatMapCellRect';
import HeatMapCellCircle from './HeatMapCellCircle';

var HeatMap = function HeatMap(_ref) {
    var data = _ref.data,
        getIndex = _ref.getIndex,
        keys = _ref.keys,
        cellWidth = _ref.cellWidth,
        cellHeight = _ref.cellHeight,
        sizeScale = _ref.sizeScale,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        offsetX = _ref.offsetX,
        offsetY = _ref.offsetY,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        cellShape = _ref.cellShape,
        cellOpacity = _ref.cellOpacity,
        cellBorderWidth = _ref.cellBorderWidth,
        getCellBorderColor = _ref.getCellBorderColor,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        getLabelTextColor = _ref.getLabelTextColor,
        theme = _ref.theme,
        colorScale = _ref.colorScale,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var Cell = void 0;
    if (cellShape === 'rect') {
        Cell = HeatMapCellRect;
    } else if (cellShape === 'circle') {
        Cell = HeatMapCellCircle;
    } else {
        Cell = cellShape;
    }

    var nodes = data.reduce(function (acc, d) {
        keys.forEach(function (key) {
            acc.push({
                key: key + '.' + getIndex(d),
                xKey: key,
                yKey: getIndex(d),
                x: xScale(key),
                y: yScale(getIndex(d)),
                width: sizeScale ? Math.min(sizeScale(d[key]) * cellWidth, cellWidth) : cellWidth,
                height: sizeScale ? Math.min(sizeScale(d[key]) * cellHeight, cellHeight) : cellHeight,
                value: d[key],
                color: colorScale(d[key])
            });
        });

        return acc;
    }, []);

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var cells = void 0;
    if (animate === true) {
        cells = React.createElement(
            TransitionMotion,
            {
                styles: nodes.map(function (node) {
                    return {
                        key: node.key,
                        data: node,
                        style: _extends({
                            x: spring(node.x, motionProps),
                            y: spring(node.y, motionProps),
                            width: spring(node.width, motionProps),
                            height: spring(node.height, motionProps)
                        }, colorMotionSpring(node.color, motionProps))
                    };
                })
            },
            function (interpolatedStyles) {
                return React.createElement(
                    'g',
                    null,
                    interpolatedStyles.map(function (_ref2) {
                        var key = _ref2.key,
                            style = _ref2.style,
                            node = _ref2.data;

                        var color = getInterpolatedColor(style);

                        return React.createElement(Cell, {
                            key: key,
                            value: node.value,
                            x: style.x,
                            y: style.y,
                            width: Math.max(style.width, 0),
                            height: Math.max(style.height, 0),
                            color: color,
                            opacity: cellOpacity,
                            borderWidth: cellBorderWidth,
                            borderColor: getCellBorderColor(_extends({}, node, { color: color })),
                            textColor: getLabelTextColor(_extends({}, node, { color: color }))
                        });
                    })
                );
            }
        );
    } else {
        cells = nodes.map(function (node) {
            return React.createElement(Cell, {
                key: node.key,
                value: node.value,
                x: node.x,
                y: node.y,
                width: node.width,
                height: node.height,
                color: node.color,
                opacity: cellOpacity,
                borderWidth: cellBorderWidth,
                borderColor: getCellBorderColor(node),
                textColor: getLabelTextColor(node)
            });
        });
    }

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;

            return React.createElement(
                SvgWrapper,
                {
                    width: outerWidth,
                    height: outerHeight,
                    margin: Object.assign({}, margin, {
                        top: margin.top + offsetY,
                        left: margin.left + offsetX
                    })
                },
                React.createElement(Grid, _extends({
                    theme: theme,
                    width: width - offsetX * 2,
                    height: height - offsetY * 2,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                React.createElement(Axes, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                cells
            );
        }
    );
};

HeatMap.propTypes = HeatMapPropTypes;

export default enhance(HeatMap);